package ru.dbotthepony.kstarbound.defs.actor

import ru.dbotthepony.kommons.io.writeBinaryString
import ru.dbotthepony.kommons.io.writeStruct2d
import ru.dbotthepony.kstarbound.io.readInternedString
import ru.dbotthepony.kstarbound.io.readVector2d
import ru.dbotthepony.kstarbound.math.vector.Vector2d
import ru.dbotthepony.kstarbound.json.builder.JsonFactory
import java.io.DataInputStream
import java.io.DataOutputStream

// this is because personality data structure is used inconsistently across original source code
interface IPersonality {
	val idle: String
	val armIdle: String
	val headOffset: Vector2d
	val armOffset: Vector2d
}

@JsonFactory(asList = 1)
data class Personality(
	override val idle: String,
	override val armIdle: String,
	override val headOffset: Vector2d,
	override val armOffset: Vector2d
) : IPersonality {
	/**
	 * new protocol only
	 */
	constructor(stream: DataInputStream) : this(
		stream.readInternedString(),
		stream.readInternedString(),
		stream.readVector2d(),
		stream.readVector2d(),
	)

	/**
	 * new protocol only
	 */
	fun write(stream: DataOutputStream) {
		stream.writeBinaryString(idle)
		stream.writeBinaryString(armIdle)
		stream.writeStruct2d(headOffset)
		stream.writeStruct2d(armOffset)
	}

	companion object {
		val ORIGINAL_ENGINE_VERY_COMPATIBLE_COMPATIBILITY = Personality("", "", Vector2d.ZERO, Vector2d.ZERO)
	}
}
