package ru.dbotthepony.kstarbound.defs.actor.behavior

import com.google.gson.JsonElement
import com.google.gson.JsonNull
import com.google.gson.TypeAdapter
import com.google.gson.annotations.JsonAdapter
import com.google.gson.stream.JsonReader
import com.google.gson.stream.JsonWriter
import ru.dbotthepony.kommons.gson.contains
import ru.dbotthepony.kommons.gson.value
import ru.dbotthepony.kstarbound.json.popObject

/**
 * [key] - references earlier value
 *
 * [value] - carries a value
 */
@JsonAdapter(NodeParameterValue.Adapter::class)
data class NodeParameterValue(val key: String?, val value: JsonElement?) {
	init {
		require(key != null || value != null) { "Both key and value are nulls" }
	}

	override fun toString(): String {
		if (key != null)
			return "key=$key"
		else
			return "value=$value"
	}

	class Adapter : TypeAdapter<NodeParameterValue>() {
		override fun write(out: JsonWriter, value: NodeParameterValue) {
			if (value.key != null) {
				out.name("key")
				out.value(value.key)
			} else {
				out.name("value")
				out.value(value.value)
			}
		}

		override fun read(`in`: JsonReader): NodeParameterValue {
			val json = `in`.popObject()

			if ("key" in json) {
				return NodeParameterValue(json["key"].asString, null)
			} else {
				return NodeParameterValue(null, json["value"] ?: JsonNull.INSTANCE)
			}
		}
	}
}