package ru.dbotthepony.kstarbound.defs.world

import com.google.gson.JsonObject
import ru.dbotthepony.kommons.util.Either
import ru.dbotthepony.kommons.util.XXHash64
import ru.dbotthepony.kstarbound.Registry
import ru.dbotthepony.kstarbound.collect.WeightedList
import ru.dbotthepony.kstarbound.defs.monster.MonsterTypeDefinition
import ru.dbotthepony.kstarbound.json.builder.JsonFactory
import ru.dbotthepony.kstarbound.math.vector.Vector2d
import ru.dbotthepony.kstarbound.math.vector.Vector2i

@JsonFactory
data class SpawnType(
	val name: String,
	val dayLevelAdjustment: Vector2d = Vector2d.ZERO,
	val nightLevelAdjustment: Vector2d = Vector2d.ZERO,

	val monsterType: Either<WeightedList<Registry.Ref<MonsterTypeDefinition>>, Registry.Ref<MonsterTypeDefinition>>,
	val monsterParameters: JsonObject = JsonObject(),
	val spawnParameters: SpawnParameters = SpawnParameters(),
	val groupSize: Vector2i = Vector2i.POSITIVE_XY,
	val spawnChance: Double,
	// tard
	@Deprecated("Raw property", replaceWith = ReplaceWith("this.actualSeedMix"))
	val seedMix: Long? = null,
) {
	val actualSeedMix: Long

	init {
		if (monsterType.isLeft && monsterType.left().isEmpty)
			throw IllegalArgumentException("monsterType is empty")

		if (seedMix != null) {
			actualSeedMix = seedMix
		} else {
			val hasher = XXHash64()
			hasher.update(name.toByteArray(Charsets.UTF_8))
			actualSeedMix = hasher.digestAsLong()
		}
	}
}