package ru.dbotthepony.kstarbound.io

import java.io.Closeable
import java.sql.Connection
import java.sql.PreparedStatement

class SQLSavepoint(connection: Connection, name: String) : Closeable {
	private val begin: PreparedStatement
	private val finish: PreparedStatement
	private val rollback: PreparedStatement

	init {
		check('"' !in name) { "Invalid identifier: $name" }

		begin = connection.prepareStatement("""
			SAVEPOINT "$name"
		""".trimIndent())

		finish = connection.prepareStatement("""
			RELEASE SAVEPOINT "$name"
		""".trimIndent())

		rollback = connection. prepareStatement("""
			ROLLBACK TO SAVEPOINT "$name"
		""".trimIndent())
	}

	fun execute(block: () -> Unit) {
		try {
			begin.execute()
			block.invoke()
			finish.execute()
		} catch (err: Throwable) {
			rollback.execute()
			throw err
		}
	}

	override fun close() {
		begin.close()
		finish.close()
		rollback.close()
	}
}