package ru.dbotthepony.kstarbound.lua

import org.apache.logging.log4j.LogManager
import ru.dbotthepony.kstarbound.Starbound

class LuaUpdateComponent(val lua: LuaThread, val name: Any) {
	var stepCount = 1.0
	private var steps = 0.0
	private var lastType = LuaType.FUNCTION

	private fun updateDt(args: LuaThread.ArgStack): Int {
		args.lua.push(stepCount * Starbound.TIMESTEP)
		return 1
	}

	private fun setUpdateDelta(args: LuaThread.ArgStack): Int {
		stepCount = args.nextDouble()
		return 0
	}

	init {
		lua.pushTable()
		lua.dup()
		lua.storeGlobal("script")

		lua.setTableValue("updateDt", ::updateDt)
		lua.setTableValue("setUpdateDelta", ::setUpdateDelta)

		lua.pop()
	}

	fun update(delta: Double, preRun: () -> Unit) {
		if (stepCount == 0.0)
			return

		steps += delta / Starbound.TIMESTEP

		if (steps >= stepCount) {
			steps %= stepCount

			val type = lua.loadGlobal("update")

			if (type != lastType) {
				lastType = type

				if (type != LuaType.FUNCTION) {
					LOGGER.warn("Lua environment for $name has $type as global 'update', script update wasn't called")
				}
			}

			if (type == LuaType.FUNCTION) {
				preRun()
				lua.push(stepCount * Starbound.TIMESTEP)
				lua.call(1)
			} else {
				lua.pop()
			}
		}
	}

	fun update(delta: Double) {
		return update(delta) {}
	}

	companion object {
		private val LOGGER = LogManager.getLogger()
	}
}
