package ru.dbotthepony.kstarbound.lua.userdata

import ru.dbotthepony.kstarbound.lua.LuaHandle
import ru.dbotthepony.kstarbound.lua.LuaThread
import java.util.concurrent.CancellationException
import java.util.concurrent.CompletableFuture
import java.util.concurrent.CompletionException

/**
 * [isLocal] tells us whenever result() should block current thread if [future] is not yet fulfilled
 * (to retain original behavior for "locally async" calls)
 *
 * god damn it.
 */
class LuaFuture<T : Any>(val future: CompletableFuture<T>, val isLocal: Boolean, val handler: LuaThread.(T) -> Int) {
	fun finished(args: LuaThread.ArgStack): Int {
		args.lua.push(future.isDone)
		return 1
	}

	fun succeeded(args: LuaThread.ArgStack): Int {
		args.lua.push(future.isDone && !future.isCompletedExceptionally)
		return 1
	}

	fun failed(args: LuaThread.ArgStack): Int {
		args.lua.push(future.isCompletedExceptionally)
		return 1
	}

	fun result(args: LuaThread.ArgStack): Int {
		try {
			if (future.isCompletedExceptionally) {
				return 0
			} else if (isLocal) {
				handler(args.lua, future.join())
				return 1
			} else {
				val result = future.getNow(null) ?: return 0
				return handler(args.lua, result)
			}
		} catch (err: CompletionException) {
			return 0
		} catch (err: CancellationException) {
			return 0
		}
	}

	fun error(args: LuaThread.ArgStack): Int {
		// this is slow, but we can't get Exception out of CompletableFuture without latter throwing former
		try {
			if (isLocal) {
				future.join()
			} else {
				future.getNow(null)
			}

			return 0
		} catch (err: CompletionException) {
			args.lua.push(err.message ?: "internal error")
			return 1
		} catch (err: CancellationException) {
			args.lua.push(err.message ?: "internal error")
			return 1
		}
	}

	companion object {
		fun initializeHandle(lua: LuaThread): LuaHandle {
			lua.pushTable()
			val handle = lua.createHandle("LuaFuture")

			lua.pushBinding("finished", LuaFuture<*>::finished)
			lua.pushBinding("succeeded", LuaFuture<*>::succeeded)
			lua.pushBinding("failed", LuaFuture<*>::failed)
			lua.pushBinding("result", LuaFuture<*>::result)
			lua.pushBinding("error", LuaFuture<*>::error)

			lua.pop()

			return handle
		}
	}
}

fun LuaThread.push(value: LuaFuture<*>) {
	pushTable()

	push("__index")
	push(commonHandles.future)
	setTableValue()

	pushObject(value)
}
