package ru.dbotthepony.kstarbound.network.packets

import ru.dbotthepony.kstarbound.client.ClientConnection
import ru.dbotthepony.kstarbound.defs.DamageData
import ru.dbotthepony.kstarbound.network.Connection
import ru.dbotthepony.kstarbound.network.IClientPacket
import ru.dbotthepony.kstarbound.network.IServerPacket
import ru.dbotthepony.kstarbound.server.ServerConnection
import java.io.DataInputStream
import java.io.DataOutputStream

class DamageRequestPacket(val inflictor: Int, val target: Int, val request: DamageData) : IServerPacket, IClientPacket {
	override fun write(stream: DataOutputStream, isLegacy: Boolean) {
		stream.writeInt(inflictor)
		stream.writeInt(target)
		request.write(stream, isLegacy)
	}

	val destinationConnection: Int
		get() = Connection.connectionForEntityID(target)

	override suspend fun play(connection: ServerConnection) {
		connection.enqueueAndSuspend {
			pushRemoteDamageRequest(this@DamageRequestPacket)
		}
	}

	override suspend fun play(connection: ClientConnection) {
		connection.enqueue {
			world?.pushRemoteDamageRequest(this@DamageRequestPacket)
		}
	}

	companion object {
		fun read(stream: DataInputStream, isLegacy: Boolean): DamageRequestPacket {
			val inflictor = stream.readInt()
			val target = stream.readInt()
			val data = DamageData(stream, isLegacy, inflictor)
			return DamageRequestPacket(inflictor, target, data)
		}
	}
}
