package ru.dbotthepony.kstarbound.network.packets.serverbound

import com.google.gson.JsonObject
import it.unimi.dsi.fastutil.bytes.ByteArrayList
import it.unimi.dsi.fastutil.io.FastByteArrayInputStream
import ru.dbotthepony.kommons.io.readByteArray
import ru.dbotthepony.kommons.io.writeByteArray
import ru.dbotthepony.kstarbound.Registries
import ru.dbotthepony.kstarbound.defs.EntityType
import ru.dbotthepony.kstarbound.io.readInternedString
import ru.dbotthepony.kstarbound.json.readJsonElement
import ru.dbotthepony.kstarbound.network.IServerPacket
import ru.dbotthepony.kstarbound.server.ServerConnection
import ru.dbotthepony.kstarbound.world.entities.ItemDropEntity
import ru.dbotthepony.kstarbound.world.entities.tile.WorldObject
import java.io.DataInputStream
import java.io.DataOutputStream

// similar to EntityCreatePacket, except client transfers ownership of entity to server
class SpawnEntityPacket(val entityType: EntityType, val storeData: ByteArrayList, val firstNetState: ByteArrayList,) : IServerPacket {
	constructor(stream: DataInputStream, isLegacy: Boolean) : this(
		EntityType.entries[stream.readUnsignedByte()],
		ByteArrayList.wrap(stream.readByteArray()),
		ByteArrayList.wrap(stream.readByteArray()),
	)

	override fun write(stream: DataOutputStream, isLegacy: Boolean) {
		stream.writeByte(entityType.ordinal)
		stream.writeByteArray(storeData.elements(), 0, storeData.size)
		stream.writeByteArray(firstNetState.elements(), 0, firstNetState.size)
	}

	override suspend fun play(connection: ServerConnection) {
		if (!entityType.canBeSpawnedByClient) {
			connection.disconnect("Illegal entity type to be spawned by client: $entityType")
			return
		}

		val entity = entityType.fromNetwork(DataInputStream(FastByteArrayInputStream(storeData.elements(), 0, storeData.size)), connection.isLegacy) ?: return
		entity.networkGroup.read(firstNetState, isLegacy = connection.isLegacy)

		connection.enqueueAndSuspend {
			entity.joinWorld(this)
		}
	}
}