package ru.dbotthepony.kstarbound.world

import ru.dbotthepony.kstarbound.io.StreamCodec
import ru.dbotthepony.kstarbound.math.vector.Vector2d
import ru.dbotthepony.kstarbound.json.builder.IStringSerializable

// uint8_t
enum class Direction(val normal: Vector2d, override val jsonName: String, val numericalValue: Long, val isRight: Boolean, val isLeft: Boolean) : IStringSerializable {
	LEFT(Vector2d.NEGATIVE_X, "left", -1L, false, true) {
		override val opposite: Direction
			get() = RIGHT
	},
	RIGHT(Vector2d.POSITIVE_X, "right", 1L, true, false) {
		override val opposite: Direction
			get() = LEFT
	};

	abstract val opposite: Direction

	operator fun unaryPlus() = opposite

	override fun match(name: String): Boolean {
		return name.lowercase() == jsonName
	}

	companion object {
		val CODEC = StreamCodec.Enum(Direction::class.java)

		fun valueOf(number: Number): Direction? {
			val value = number.toDouble()

			if (value == 0.0) {
				return null
			} else if (value < 0.0) {
				return LEFT
			} else {
				return RIGHT
			}
		}

		fun valueOf(value: Long): Direction? {
			if (value == 0L) {
				return null
			} else if (value < 0L) {
				return LEFT
			} else {
				return RIGHT
			}
		}

		fun valueOf(value: Double): Direction? {
			if (value == 0.0) {
				return null
			} else if (value < 0.0) {
				return LEFT
			} else {
				return RIGHT
			}
		}
	}
}
